package com.os.operando.advertisingid

import android.content.BroadcastReceiver
import android.content.Context
import com.google.android.gms.ads.identifier.AdvertisingIdClient
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.plugin.common.BinaryMessenger
import io.flutter.plugin.common.EventChannel
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import io.flutter.plugin.common.PluginRegistry.Registrar
import kotlin.concurrent.thread


class AdvertisingIdPlugin(private val registrar: Registrar) : FlutterPlugin, MethodCallHandler {

    companion object {
        @JvmStatic
        fun registerWith(registrar: Registrar) {
            val instance = AdvertisingIdPlugin(registrar)
            instance.onAttachedToEngine(registrar.messenger())
        }
    }

    override fun onAttachedToEngine(binding: FlutterPlugin.FlutterPluginBinding) {
        onAttachedToEngine(binding.binaryMessenger)
    }

    override fun onDetachedFromEngine(binding: FlutterPlugin.FlutterPluginBinding) {
    }

    override fun onMethodCall(call: MethodCall, result: Result) {
        when (call.method) {
            "getAdvertisingId" -> thread {
                try {
                    val id = AdvertisingIdClient.getAdvertisingIdInfo(registrar.context()).id
                    registrar.activity().runOnUiThread {
                        result.success(id)
                    }
                } catch (e: Exception) {
                    registrar.activity().runOnUiThread {
                        result.error(e.javaClass.canonicalName, e.localizedMessage, null)
                    }
                }
            }
            "isLimitAdTrackingEnabled" -> thread {
                try {
                    val isLimitAdTrackingEnabled = AdvertisingIdClient.getAdvertisingIdInfo(registrar.context()).isLimitAdTrackingEnabled
                    registrar.activity().runOnUiThread {
                        result.success(isLimitAdTrackingEnabled)
                    }
                } catch (e: Exception) {
                    registrar.activity().runOnUiThread {
                        result.error(e.javaClass.canonicalName, e.localizedMessage, null)
                    }
                }
            }
            else -> result.notImplemented()
        }
    }

    private fun onAttachedToEngine(messenger: BinaryMessenger) {
        val methodChannel = MethodChannel(messenger, "advertising_id")
        methodChannel.setMethodCallHandler(this)
    }
}
